﻿# Airfoil Metadata Protocol

## Introduction

The Airfoil Metadata Protocol is designed to allow applications to provide Airfoil with track
metadata and remote control capabilities. 

The protocol uses named pipes for exchanging simple request/response messages. Using named
pipes allows us to provide the broadest language support possible without resorting to something
horrific like COM.

## Message Framing

All messages are sent in the following format:

    length;message

The length field indicates the length of the message segment in UTF-8 encoded bytes.


## Named Pipe Creation

It is the responsibility of the source application to create the named pipe. When Airfoil
is intercepting audio from an application, it will periodically look for a named pipe called:

    \\.\pipe\{process id}_airfoil_metadata

Where {process id} is the process id of the application's main process. The main process id
should be used even if it is a child process that is responsbile for creating meatadata or
handling metadata requests.

Thus if your application's main process has a process id of 8675, the named pipe you create
should be called:

    \\.\pipe\8675_airfoil_metadata

Note that if you are using the .NET framework's System.IO.Pipes classes to create your named
pipe, the "\\.\pipe\" portion of the pipe name is handled automatically by the framework. 
    
## Messages

Once the named pipe connection has been made, the application should begin reading messages sent
by Airfoil. The first messages sent across the pipe will be supportsRemoteControl and providesTrackData.

If the application responds to supportsRemoteControl with "true", then it must also respond to
remotePlayPause, remoteTrackNext, and remoteTrackPrevious, even if it doesn't take any action in
response to those messages.

If the application responds to providesTrackData with "true", then it must also respond to
requestTrackTitle, requestTrackArtist, requestTrackAlbum, and requestAlbumArt, even if it only
responds to those messages with an empty string. 

### supportsRemoteControl 

Indicates whether or not an application responds to remote commands.
Valid responses: true, false

### providesTrackData

Indicates whether or not an application provides track metadata.
Valid resoponses: true, false

### remotePlayPause

Signals the application to toggle play/pause.
A response of OK indicates success. Any other response is interpreted as failure.

### remoteTrackNext

Signals the application to advance to the next track.
A response of OK indicates success. Any other response is interpreted as failure.
    
### remoteTrackPrevious

Signals the application to advance to the previous track.
A response of OK indicates success. Any other response is interpreted as failure.

### requestTrackTitle

A request for the title of the current track.
Valid responses include the name of the current track, or an empty string to indicate no data. 
 
### requestTrackArtist

A request for the artist name of the current track.
Valid responses include the name of the current track's artist, or an empty string to indicate no
data.

### requestTrackAlbum

A request for the album name of the current track.
Valid responses include the name of the current track's album, or an empty string to indicate no
data.

### requestAlbumArt

A request for the album art image associated with the current track.
Valid responses include a base-64 encoded PNG representation of the album art, or an empty string
to indicate no data.

## Example message exchange:

21;supportsRemoteControl\n
4;true\n
17;providesTrackData\n
4;true\n
17;requestTrackTitle\n
5;Happy\n
15;remoteTrackNext\n
2;OK\n